@@ -2,7 +2,7 @@
  *
  *  AT chat library with GLib integration
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -35,6 +35,7 @@
 #include "crc-ccitt.h"
 #include "ringbuffer.h"
 #include "gatio.h"
+#include "gatutil.h"
 #include "gathdlc.h"
 
 #define BUFFER_SIZE	(2 * 2048)
@@ -69,6 +70,8 @@ struct _GAtHDLC {
 	int record_fd;
 	gboolean in_read_handler;
 	gboolean destroyed;
+	gboolean wakeup_sent;
+	gboolean start_frame_marker;
 	gboolean no_carrier_detect;
 	GAtSuspendFunc suspend_func;
 	gpointer suspend_data;
@@ -77,7 +80,8 @@ struct _GAtHDLC {
 	guint num_plus;
 };
 
-static void hdlc_record(int fd, gboolean in, guint8 *data, guint16 length)
+static inline void hdlc_record(GAtHDLC *hdlc, gboolean in,
+					guint8 *data, guint16 length)
 {
 	guint16 len = htons(length);
 	guint32 ts;
@@ -85,10 +89,13 @@ static void hdlc_record(int fd, gboolean in, guint8 *data, guint16 length)
 	unsigned char id;
 	int err;
 
-	if (fd < 0)
+	g_at_util_debug_hexdump(in, data, length,
+					hdlc->debugf, hdlc->debug_data);
+
+	if (hdlc->record_fd < 0)
 		return;
 
-	if (len == 0)
+	if (length == 0)
 		return;
 
 	gettimeofday(&now, NULL);
@@ -96,25 +103,25 @@ static void hdlc_record(int fd, gboolean in, guint8 *data, guint16 length)
 
 	id = 0x07;
 
-	err = write(fd, &id, 1);
+	err = write(hdlc->record_fd, &id, 1);
 	if (err < 0)
 		return;
 
-	err = write(fd, &ts, 4);
+	err = write(hdlc->record_fd, &ts, 4);
 	if (err < 0)
 		return;
 
 	id = in ? 0x02 : 0x01;
 
-	err = write(fd, &id, 1);
+	err = write(hdlc->record_fd, &id, 1);
 	if (err < 0)
 		return;
 
-	err = write(fd, &len, 2);
+	err = write(hdlc->record_fd, &len, 2);
 	if (err < 0)
 		return;
 
-	err = write(fd, data, length);
+	err = write(hdlc->record_fd, data, length);
 	if (err < 0)
 		return;
 }
@@ -260,7 +267,7 @@ static void new_bytes(struct ring_buffer *rbuf, gpointer user_data)
 			return;
 	}
 
-	hdlc_record(hdlc->record_fd, TRUE, buf, wrap);
+	hdlc_record(hdlc, TRUE, buf, wrap);
 
 	hdlc->in_read_handler = TRUE;
 
@@ -308,7 +315,7 @@ static void new_bytes(struct ring_buffer *rbuf, gpointer user_data)
 
 		if (pos == wrap) {
 			buf = ring_buffer_read_ptr(rbuf, pos);
-			hdlc_record(hdlc->record_fd, TRUE, buf, len - wrap);
+			hdlc_record(hdlc, TRUE, buf, len - wrap);
 		}
 	}
 
@@ -324,7 +331,6 @@ out:
 GAtHDLC *g_at_hdlc_new_from_io(GAtIO *io)
 {
 	GAtHDLC *hdlc;
-	unsigned char *buf;
 	struct ring_buffer* write_buffer;
 
 	if (io == NULL)
@@ -353,11 +359,6 @@ GAtHDLC *g_at_hdlc_new_from_io(GAtIO *io)
 
 	g_queue_push_tail(hdlc->write_queue, write_buffer);
 
-	/* Write an initial 0x7e as wakeup character */
-	buf = ring_buffer_write_ptr(write_buffer, 0);
-	*buf = HDLC_FLAG;
-	ring_buffer_write_advance(write_buffer, 1);
-
 	hdlc->decode_buffer = g_try_malloc(BUFFER_SIZE);
 	if (!hdlc->decode_buffer)
 		goto error;
@@ -376,8 +377,7 @@ error:
 	if (write_buffer)
 		ring_buffer_free(write_buffer);
 
-	if (hdlc->decode_buffer)
-		g_free(hdlc->decode_buffer);
+	g_free(hdlc->decode_buffer);
 
 	g_free(hdlc);
 
@@ -482,7 +482,7 @@ static gboolean can_write_data(gpointer data)
 	buf = ring_buffer_read_ptr(write_buffer, 0);
 
 	bytes_written = g_at_io_write(hdlc->io, (gchar *) buf, len);
-	hdlc_record(hdlc->record_fd, FALSE, buf, bytes_written);
+	hdlc_record(hdlc, FALSE, buf, bytes_written);
 	ring_buffer_drain(write_buffer, bytes_written);
 
 	if (ring_buffer_len(write_buffer) > 0)
@@ -560,6 +560,24 @@ gboolean g_at_hdlc_send(GAtHDLC *hdlc, const unsigned char *data, gsize size)
 	i = 0;
 	buf = ring_buffer_write_ptr(write_buffer, 0);
 
+	if (hdlc->start_frame_marker == TRUE) {
+		/* Protocol requires 0x7e as start marker */
+		if (pos + 1 > avail)
+			return FALSE;
+
+		*buf++ = HDLC_FLAG;
+		pos++;
+
+		if (pos == wrap)
+			buf = ring_buffer_write_ptr(write_buffer, pos);
+	} else if (hdlc->wakeup_sent == FALSE) {
+		/* Write an initial 0x7e as wakeup character */
+		*buf++ = HDLC_FLAG;
+		pos++;
+
+		hdlc->wakeup_sent = TRUE;
+	}
+
 	while (pos < avail && i < size) {
 		if (escape == TRUE) {
 			fcs = HDLC_FCS(fcs, data[i]);
@@ -613,6 +631,7 @@ gboolean g_at_hdlc_send(GAtHDLC *hdlc, const unsigned char *data, gsize size)
 	if (pos + 1 > avail)
 		return FALSE;
 
+	/* Add 0x7e as end marker */
 	*buf = HDLC_FLAG;
 	pos++;
 
@@ -623,6 +642,14 @@ gboolean g_at_hdlc_send(GAtHDLC *hdlc, const unsigned char *data, gsize size)
 	return TRUE;
 }
 
+void g_at_hdlc_set_start_frame_marker(GAtHDLC *hdlc, gboolean marker)
+{
+	if (hdlc == NULL)
+		return;
+
+	hdlc->start_frame_marker = marker;
+}
+
 void g_at_hdlc_set_no_carrier_detect(GAtHDLC *hdlc, gboolean detect)
 {
 	if (hdlc == NULL)
